@@ -28,19 +28,6 @@
 
 from fstd2nc.stdout import _, info, warn, error
 
-# Override default dtype for "binary" (datyp 0) data.
-# This is probably float32 data, not integer.
-# See (internal) mailing list discussion at http://internallists.cmc.ec.gc.ca/pipermail/python-rpn/2015-February/000010.html where this problem is discussed.
-# Also see examples on (internal) wiki dealing with this type of data:
-# https://wiki.cmc.ec.gc.ca/wiki/Python-RPN/2.0/examples#Example_4:_Plot_RPN_STD_field_on_an_X-grid
-# https://wiki.cmc.ec.gc.ca/wiki/Talk:Python-RPN/2.0/examples#Plot_GIOPS_Forecast_Data_with_Basemap
-def dtype_fst2numpy (datyp, nbits=None):
-  from rpnpy.librmn.fstd98 import dtype_fst2numpy
-  if datyp == 0:
-    warn (_("Raw binary records detected.  The values may not be properly decoded if you're opening on a different platform."))
-    datyp = 5
-  return dtype_fst2numpy(datyp,nbits)
-
 # Indicate if pandas should be used or not.
 _pandas_needed = False
 def _use_pandas ():
@@ -215,25 +202,11 @@ try:
 except ImportError:
   _ProgressBar = _FakeBar
 
-
-# Define a lock for controlling threaded access to the RPN file(s).
-# This is necessary because we can only open a limited number of files
-# at a time, so need to control which ones are opened and available in
-# a thread-safe way.
-from threading import RLock
-_lock = RLock()
-del RLock
-
-# Define a class for encoding / decoding FSTD data.
+# Define a class for encoding / decoding the data.
 # Each step is placed in its own "mixin" class, to make it easier to patch in 
-# new behaviour if more exotic FSTD files are encountered in the future.
+# new behaviour if more exotic files are encountered in the future.
 class BufferBase (object):
 
-  # Keep a reference to fstd98 so it's available during cleanup.
-  try:
-    from rpnpy.librmn import fstd98 as _fstd98
-  except ImportError: pass
-
   # Names of records that should be kept separate (never grouped into
   # multidimensional arrays).
   _meta_records = ()
@@ -242,9 +215,15 @@ class BufferBase (object):
   # variable.
   _maybe_meta_records = ()
 
-  # Attributes which could potentially be used as axes.
+  # Attributes which could potentially be used as outer axes.
+  # The values from the attribute will become the axis values.
   _outer_axes = ()
 
+  # Attributes which could be used as inner axes.
+  # The values represent the *length* of the axis, which will have no
+  # coordinate values by default.
+  _inner_axes = ()
+
   # Attributes which could be used as auxiliary coordinates for the outer
   # axes.  The dictionary keys are the outer axis names, and the values are
   # a list of columns which can act as coordinates.
@@ -253,17 +232,16 @@ class BufferBase (object):
   del OrderedDict
 
   # Attributes which uniquely identify a variable.
-  # Note: nomvar should always be the first attribute
-  _var_id = ('nomvar','ni','nj','nk')
+  _var_id = ()
 
   # Similar to above, but a human-readable version of the id.
   # Could be used for appending suffixes to variable names to make them unique.
   # Uses string formatting operations on the variable metadata.
-  _human_var_id = ('%(nomvar)s', '%(ni)sx%(nj)s', '%(nk)sL')
+  _human_var_id = ()
 
   # Record parameters which should not be used as nc variable attributes.
   # (They're either internal to the file, or part of the data, not metadata).
-  _ignore_atts = ('file_id','swa','lng','dltf','ubc','xtra1','xtra2','xtra3','key','shape','d')
+  _ignore_atts = ('file_id','key','name','address','length','dtype','selected')
 
   # Define any command-line arguments for reading FSTD files.
   @classmethod
@@ -276,29 +254,17 @@ class BufferBase (object):
     _('Display a progress bar during the conversion, if the "progress" module is installed.')
     group.add_argument('--progress', action='store_true', default=stdout.isatty(), help=SUPPRESS)
     group.add_argument('--no-progress', action='store_false', dest='progress', help=_('Disable the progress bar.'))
-    group = parser.add_mutually_exclusive_group()
-    group.add_argument('--minimal-metadata', action='store_true', default=True, help=_("Don't include RPN record attributes and other internal information in the output metadata.")+" "+_("This is the default behaviour."))
-    group.add_argument('--rpnstd-metadata', action='store_false', dest='minimal_metadata', help=_("Include all RPN record attributes in the output metadata."))
-    group.add_argument('--rpnstd-metadata-list', metavar='nomvar,...', help=_("Specify a minimal set of RPN record attributes to include in the output file."))
-    parser.add_argument('--ignore-typvar', action='store_true', help=_('Tells the converter to ignore the typvar when deciding if two records are part of the same field.  Default is to split the variable on different typvars.'))
-    parser.add_argument('--ignore-etiket', action='store_true', help=_('Tells the converter to ignore the etiket when deciding if two records are part of the same field.  Default is to split the variable on different etikets.'))
     parser.add_argument('--serial', action='store_true', help=_('Disables multithreading/multiprocessing.  Useful for resource-limited machines.'))
+    group = parser.add_mutually_exclusive_group()
+    group.add_argument('--minimal-metadata', action='store_true', default=True, help=_("Don't include internal record attributes and other internal information in the output metadata.")+" "+_("This is the default behaviour."))
+    group.add_argument('--internal-metadata','--rpnstd-metadata', action='store_false', dest='minimal_metadata', help=_("Include all internal record attributes in the output metadata."))
+    group.add_argument('--metadata-list','--rpnstd-metadata-list', metavar='nomvar,...', help=_("Specify a minimal set of internal record attributes to include in the output file."))
 
   # Do some checks on the command-line arguments after parsing them.
   @classmethod
   def _check_args (cls, parser, args):
     return  # Nothing to check here.
 
-  # Clean up a buffer (close any attached files, etc.)
-  def __del__ (self):
-    try:
-      self._close()
-      from rpnpy.librmn.fstd98 import fstcloseall
-      fstcloseall(self._meta_funit)
-    except (ImportError, AttributeError):
-      pass  # May fail if Python is doing a final cleanup of everything.
-            # Or, if buffer wasn't fully initialized yet.
-
   # Extract metadata from a particular header.
   def _get_header_atts (self, header):
     for n,v in header.items():
@@ -310,90 +276,18 @@ class BufferBase (object):
         v = v.strip()
       yield (n,v)
 
-  # Open the specified file (by index)
-  def _open (self, file_id):
-    from rpnpy.librmn.base import fnom
-    from rpnpy.librmn.fstd98 import fstouv
-    from rpnpy.librmn.const import FST_RO
-    from rpnpy.librmn import librmn
-    opened_file_id = getattr(self,'_opened_file_id',-1)
-    # Check if this file already opened.
-    if opened_file_id == file_id:
-      return self._opened_funit
-    # Close any open files before continuing.
-    self._close()
-    filename = self._files[file_id]
-    # Open the file.
-    self._opened_file_id = file_id
-    self._opened_funit = fnom(filename,FST_RO)
-    fstouv(self._opened_funit,FST_RO)
-    self._opened_librmn_index = librmn.file_index(self._opened_funit)
-    return self._opened_funit
-
-  # Close any currently opened file.
-  def _close (self):
-    from rpnpy.librmn.base import fclos
-    from rpnpy.librmn.fstd98 import fstfrm
-    opened_funit = getattr(self,'_opened_funit',-1)
-    if opened_funit >= 0:
-      fstfrm(opened_funit)
-      fclos(opened_funit)
-    self._opened_file_id = -1
-    self._opened_funit = -1
-    self._opened_librmn_index = -1
-
-  # Open metadata funit (done once during init of object).
-  def _open_meta_funit (self):
-    from rpnpy.librmn.fstd98 import fstopenall
-    from rpnpy.librmn.const import FST_RO
-    import numpy as np
-    import warnings
-    meta_mask = self._headers['ismeta'][:]
-    meta_recids = np.where(meta_mask)[0]
-    _var_id = ('nomvar','ni','nj','nk')
-    # Use the same unique parameters as regular variables.
-    # Plus, ig1,ig2,ig3,ig4.
-    # Suppress FutureWarning from numpy about doing this.  Probably benign...
-    with warnings.catch_warnings():
-      warnings.simplefilter("ignore")
-      from fstd2nc.extra import structured_array
-      headers = dict()
-      for key in list(_var_id)+['ip1','ip2','ip3','ig1','ig2','ig3','ig4']:
-        headers[key] = self._headers[key]
-      headers = structured_array(headers)
-      meta_keys = headers.data[meta_mask][list(_var_id)+['ip1','ip2','ip3','ig1','ig2','ig3','ig4']]
-    meta_keys, ind = np.unique(meta_keys, return_index=True)
-    meta_recids = meta_recids[ind]
-    # Find the files that give these unique coord records.
-    file_ids = sorted(set(self._headers['file_id'][meta_recids]))
-    filenames = [self._files[f] for f in file_ids]
-    if len(filenames) > 500:
-      error(_("Holy crap, how many coordinates do you have???"))
-    # If no coordinates found, just open the first file as a dummy file.
-    # Less error-prone than checking if _meta_funit is defined every time
-    # an FSTD function is called.
-    if len(filenames) == 0:
-      filenames = self._files[0:1]
-    # Open these files and link them together
-    self._meta_filenames = filenames  # Store this for further hacking.
-    self._meta_funit = fstopenall(filenames, FST_RO)
-        
   # Control the pickling / unpickling of BufferBase objects.
   def __getstate__ (self):
     state = self.__dict__.copy()
-    state.pop('_lock',None)  # librmn lock will be defined upon unpickling.
-    state.pop('_meta_funit',None) # Same with librmn file handles.
     return state
   def __setstate__ (self, state):
     self.__dict__.update(state)
-    self._lock = _lock
-    self._open_meta_funit()
 
 
   ###############################################
   # Basic flow for reading data
 
-  def __init__ (self, filename, _headers=None, progress=False, minimal_metadata=None, rpnstd_metadata=None, rpnstd_metadata_list=None, ignore_typvar=False, ignore_etiket=False, serial=False):
+  def __init__ (self, filename, _headers=None, progress=False, serial=False, **kwargs):
     """
     Read raw records from FSTD files, into the buffer.
     Multiple files can be read simultaneously.
@@ -401,28 +295,19 @@ class BufferBase (object):
     Parameters
     ----------
     filename : str or list
-        The RPN standard file(s) to convert.
+        The input files to convert.
     progress : bool, optional
         Display a progress bar during the conversion, if the "progress"
         module is installed.
-    rpnstd_metadata : bool, optional
-        Include all RPN record attributes in the output metadata.
-    rpnstd_metadata_list : str or list, optional
-        Specify a minimal set of RPN record attributes to include in the
-        output file.
-    ignore_typvar : bool, optional
-        Tells the converter to ignore the typvar when deciding if two
-        records are part of the same field.  Default is to split the
-        variable on different typvars.
-    ignore_etiket : bool, optional
-        Tells the converter to ignore the etiket when deciding if two
-        records are part of the same field.  Default is to split the
-        variable on different etikets.
     serial : bool, optional
         Disables multithreading/multiprocessing.  Useful for resource-limited
         machines.
+    internal_metadata : bool, optional
+        Include all internal record attributes in the output metadata.
+    metadata_list : str or list, optional
+        Specify a minimal set of internal record attributes to include in the
+        output file.
     """
-    from fstd2nc.extra import raw_headers, decode_headers
     from collections import Counter
     import numpy as np
     from glob import glob, has_magic
@@ -433,38 +318,9 @@ class BufferBase (object):
     except ImportError:
       imap = map
 
-    # Set up lock for threading.
-    # The same lock is shared for all Buffer objects, to synchronize access to
-    # librmn.
-    self._lock = _lock
-
     # Set up a progress bar for scanning the input files.
     Bar = _ProgressBar if progress is True else _FakeBar
 
-    # Set default for minimal_metadata
-    if rpnstd_metadata is not None:
-      minimal_metadata = not rpnstd_metadata
-    if minimal_metadata is None:
-      minimal_metadata = True
-    # Set default for rpnstd_metadata_list
-    if minimal_metadata is True and rpnstd_metadata_list is None:
-      rpnstd_metadata_list = ''
-    if isinstance(rpnstd_metadata_list,str):
-      rpnstd_metadata_list = rpnstd_metadata_list.replace(',',' ')
-      rpnstd_metadata_list = rpnstd_metadata_list.split()
-    if hasattr(rpnstd_metadata_list,'__len__'):
-      rpnstd_metadata_list = tuple(rpnstd_metadata_list)
-    self._rpnstd_metadata_list = rpnstd_metadata_list
-
-    if not ignore_typvar:
-      # Insert typvar value just after nomvar.
-      self._var_id = self._var_id[0:1] + ('typvar',) + self._var_id[1:]
-      self._human_var_id = self._human_var_id[0:1] + ('%(typvar)s',) + self._human_var_id[1:]
-    if not ignore_etiket:
-      # Insert etiket value just after nomvar.
-      self._var_id = self._var_id[0:1] + ('etiket',) + self._var_id[1:]
-      self._human_var_id = self._human_var_id[0:1] + ('%(etiket)s',) + self._human_var_id[1:]
-
     self._serial = serial
 
     if isinstance(filename,str):
@@ -483,17 +339,47 @@ class BufferBase (object):
         else:
           expanded_infiles.append((infile,f))
 
+    # How to handle internal metadata.
+    internal_metadata = kwargs.pop('internal_metadata',None)
+    # Check for legacy rpnstd_metadata argument
+    if internal_metadata is None:
+      internal_metadata = kwargs.pop('rpnstd_metadata',None)
+    metadata_list = kwargs.pop('metadata_list',None)
+    # Check for legacy rpnstd_metadata_list argument
+    if metadata_list is None:
+      metadata_list = kwargs.pop('rpnstd_metadata_list',None)
+    minimal_metadata = kwargs.pop('minimal_metadata',None)
+
+    # Set default for minimal_metadata
+    if internal_metadata is not None:
+      minimal_metadata = not internal_metadata
+    if minimal_metadata is None:
+      minimal_metadata = True
+    # Set default for metadata_list
+    if minimal_metadata is True and metadata_list is None:
+      metadata_list = ''
+    if isinstance(metadata_list,str):
+      metadata_list = metadata_list.replace(',',' ')
+      metadata_list = metadata_list.split()
+    if hasattr(metadata_list,'__len__'):
+      metadata_list = tuple(metadata_list)
+    self._metadata_list = metadata_list
+
+    # Should not have any unprocessed keyword arguments after this.
+    if len(kwargs) > 0:
+      error(_("Unexpected arguments: %s"%(kwargs.keys())))
+
     # Extract headers from the files.
     if len(expanded_infiles) == 1: Bar = _FakeBar
     bar = Bar(_("Inspecting input files"), suffix='%(percent)d%% (%(index)d/%(max)d)', max=len(expanded_infiles))
 
     if len(expanded_infiles) > 1 and not self._serial:
       with Pool() as p:
-        headers = p.imap (raw_headers, [f for (infile,f) in expanded_infiles])
+        headers = p.imap (self._raw_headers, [f for (infile,f) in expanded_infiles])
         headers = bar.iter(headers)
         headers = list(headers) # Start scanning.
     else:
-      headers = imap (raw_headers, [f for (infile,f) in expanded_infiles])
+      headers = imap (self._raw_headers, [f for (infile,f) in expanded_infiles])
       headers = bar.iter(headers)
       headers = list(headers) # Start scanning.
 
@@ -508,31 +394,32 @@ class BufferBase (object):
         matches[infile] += 1
         filenum = len(self._files)
         self._files.append(f)
-        file_ids.extend([filenum]*(len(headers[i])//72))
+        file_ids.extend([filenum]*len(headers[i]))
       else:
         matches[infile] += 0
 
     # Decode all the headers
     headers = [h for h in headers if h is not None]
     # Remember indices in the headers (needed for reconstructing keys)
-    indices = [range(len(h)//72) for h in headers]
+    indices = [range(len(h)) for h in headers]
     if len(headers) > 0:
       headers = np.concatenate(headers)
       indices = np.concatenate(indices)
-      headers = decode_headers(headers)
+      headers = self._decode_headers(headers)
       # Encode the keys without the file index info.
       headers['key'] = (indices % 256) | ((indices//256)<<9)
       headers['file_id'] = np.array(file_ids, dtype='int32')
+      headers['selected'] = np.ones(len(file_ids),'bool')
 
     # Check if the input entries actually matched anything.
     for infile, count in matches.items():
       if count == 0:
         if os.path.isfile(infile):
-          warn(_("'%s' is not an RPN standard file.")%infile)
+          warn(_("'%s' is not %s.")%(infile,self._format_singular))
         elif os.path.isdir(infile):
-          warn(_("Directory '%s' does not contain any RPN standard files.")%infile)
+          warn(_("Directory '%s' does not contain any %s.")%(infile,self._format_plural))
         elif has_magic(infile):
-          warn(_("No RPN standard files match '%s'.")%infile)
+          warn(_("No %s match '%s'.")%(self._format_plural,infile))
         elif not os.path.exists(infile):
           warn(_("'%s' does not exist.")%infile)
         else:
@@ -544,7 +431,7 @@ class BufferBase (object):
     elif nfiles > 10:
       global _pandas_needed
       _pandas_needed = True
-    info(_("Found %d RPN input file(s)"%nfiles))
+    info(_("Found %d %s"%(nfiles,self._format_plural)))
 
     # Add extra headers? (hack for generating Buffer objects from external
     # sources (not FSTD files)
@@ -554,27 +441,10 @@ class BufferBase (object):
       headers['file_id'] = np.zeros(len(headers['nomvar']), dtype='int32')
 
     self._headers = headers
-    self._nrecs = len(headers['nomvar'])
+    self._nrecs = max(len(self._headers[key]) for key in self._headers.keys())
 
-    # Find all unique meta (coordinate) records, and link a subset of files
-    # that provide all unique metadata records.
-    # This will make it easier to look up the meta records later.
-    meta_mask = np.zeros(self._nrecs,dtype='bool')
-    for meta_name in self._meta_records:
-      meta_name = (meta_name+b'   ')[:4]
-      meta_mask |= (self._headers['nomvar'] == meta_name)
-    for meta_name in self._maybe_meta_records:
-      meta_name = (meta_name+b'   ')[:4]
-      meta_mask |= (self._headers['nomvar'] == meta_name) & ((self._headers['ni']==1)|(self._headers['nj']==1))
 
-    # Store this metadata identification in case it's useful for some mixins.
-    self._headers['ismeta'] = np.empty(self._nrecs, dtype='bool')
-    self._headers['ismeta'][:] = meta_mask
-
-    # Make metadata records available for mixins.
-    self._open_meta_funit()
-
-  # Generate structured variables from FST records.
+  # Generate structured variables from the data records.
 
   # Normal version (without use of pandas).
   def _makevars_slow (self):
@@ -591,7 +461,7 @@ class BufferBase (object):
     records = structured_array(self._headers)
 
     # Ignore deleted / invalidated records, and coordinate records.
-    valid = (records['dltf'] == 0) & (records['ismeta'] == 0)
+    valid = (records['selected'] == True)
     records = records[valid]
     header_indices = np.where(valid)[0]
 
@@ -629,7 +499,7 @@ class BufferBase (object):
       selection = (all_var_ids == var_id)
       var_records = records[selection]
       var_record_indices = np.where(selection)[0]
-      nomvar = var_id['nomvar'].strip()
+      nomvar = var_id['name'].strip()
       nomvar = str(nomvar.decode()) # Python3: convert bytes to str.
 
       # Get the metadata for each record.
@@ -644,13 +514,13 @@ class BufferBase (object):
         # Only use attributes that are consistent across all variable records.
         if len(set(v)) > 1: continue
         v = v[0]
-        # Use regular integers for numeric types.
-        if np.can_cast(v.dtype,int):
-          v = int(v)
         # Python3: convert bytes to str.
         if isinstance(v,bytes): v = str(v.decode())
         # Trim string attributes (remove whitespace padding).
         if isinstance(v,str): v = v.strip()
+        # Use regular integers for numeric types.
+        elif np.can_cast(v.dtype,int):
+          v = int(v)
         atts[n] = v
 
       # Get the axes.
@@ -709,32 +579,37 @@ class BufferBase (object):
                            axes = list(coordaxes.values()), array = values )
           known_coords[key] = coord
         coords.append(known_coords[key])
-      if len(coords) > 0:
-        atts['coordinates'] = coords
 
       # Check if we have full coverage along all axes.
       have_data = (record_id.flatten() >= 0)
       if not np.all(have_data):
         warn (_("Missing some records for %s.")%nomvar)
 
-      # Add dummy axes for the ni,nj,nk record dimensions.
-      axes['k'] = _dim_type(name='k', length = int(var_id['nk']))
-      axes['j'] = _dim_type(name='j', length = int(var_id['nj']))
-      axes['i'] = _dim_type(name='i', length = int(var_id['ni']))
+      # Add dummy axes for the inner dimensions.
+      for n in self._inner_axes:
+        values = var_records[n]
+        # Remove missing values before continuing.
+        values = np.ma.compressed(values)
+        # Ignore dimensions that are masked out or are inconsistent.
+        # Note: inner dimensions should never be inconsistent.  They should
+        # be encoded into the var_id fields to ensure that.
+        if len(values) == 0: continue
+        length = values[0]
+        axes[n] = _dim_type(name=n, length = length)
 
       # Determine the optimal data type to use.
-      # First, find unique combos of datyp, nbits
-      # (want to minimize calls to dtype_fst2numpy).
-      datyp, nbits = zip(*np.unique(var_records.data[['datyp','nbits']]))
-      datyp = map(int,datyp)
-      nbits = map(int,nbits)
-      dtype_list = map(dtype_fst2numpy, datyp, nbits)
+      dtype_list = var_records['dtype']
       dtype = np.result_type(*dtype_list)
 
       var = _iter_type( name = nomvar, atts = atts,
                         axes = list(axes.values()),
                         dtype = dtype,
                         record_id = record_id )
+      # Add auxiliary coordinate variables through silent dependency.
+      # Don't want these in the 'coordinates' attribute because they seem to
+      # cause issues with certain netCDF decoders.
+      if len(coords) > 0:
+        var.deps = coords
       self._varlist.append(var)
 
       #TODO: Find a minimum set of partial coverages for the data.
@@ -758,11 +633,8 @@ class BufferBase (object):
     # Keep track of original dtypes (may need to re-cast later).
     original_dtypes = dict([(key,value.dtype) for key,value in self._headers.items()])
 
-    # Ignore deleted / invalidated records.
-    records = records[records['dltf']==0]
-
-    # Ignore coordinate records.
-    records = records[records['ismeta']==0]
+    # Get the subset of records destined for output.
+    records = records[records['selected']==True]
 
     # Keep track of any axes that were generated.
     known_axes = dict()
@@ -775,7 +647,7 @@ class BufferBase (object):
     self._varlist = []
     for var_id, var_records in records.groupby(list(self._var_id)):
       var_id = OrderedDict(zip(self._var_id, var_id))
-      nomvar = var_id['nomvar'].strip()
+      nomvar = var_id['name'].strip()
       nomvar = str(nomvar.decode()) # Python3: convert bytes to str.
 
       # Get the attributes, axes, and corresponding indices of each record.
@@ -872,8 +744,6 @@ class BufferBase (object):
                            array = values )
           known_coords[key] = coord
         coords.append(known_coords[key])
-      if len(coords) > 0:
-        atts['coordinates'] = coords
 
 
 
@@ -882,24 +752,42 @@ class BufferBase (object):
       if not np.all(have_data):
         warn (_("Missing some records for %s.")%nomvar)
 
-      # Add dummy axes for the ni,nj,nk record dimensions.
-      axes['k'] = _dim_type('k',int(var_id['nk']))
-      axes['j'] = _dim_type('j',int(var_id['nj']))
-      axes['i'] = _dim_type('i',int(var_id['ni']))
+      # Add dummy axes for the inner dimensions.
+      for n in self._inner_axes:
+        # [Note: this code copied from above for dealing with masked data
+        #  in pandas DataFrames]
+        #
+        # Ignore dimensions that are masked out or are inconsistent.
+        # Note: inner dimensions should never be inconsistent.  They should
+        # be encoded into the var_id fields to ensure that.
+        try:
+          if np.isnan(var_records[n].values[0]): continue
+        except TypeError:
+          if var_records[n].values[0] is None: continue
+        # Get the unique values, in order.
+        column = var_records[n].astype(original_dtypes[n])
+        ###
+        values = column.values
+        # Ignore dimensions that are masked out or are inconsistent.
+        # Note: inner dimensions should never be inconsistent.  They should
+        # be encoded into the var_id fields to ensure that.
+        if len(values) == 0: continue
+        length = values[0]
+        axes[n] = _dim_type(name=n, length = length)
 
       # Determine the optimal data type to use.
-      # First, find unique combos of datyp, nbits
-      # (want to minimize calls to dtype_fst2numpy).
-      x = var_records[['datyp','nbits']].drop_duplicates()
-      datyp = map(int,x['datyp'])
-      nbits = map(int,x['nbits'])
-      dtype_list = map(dtype_fst2numpy, datyp, nbits)
+      dtype_list = var_records['dtype']
       dtype = np.result_type(*dtype_list)
 
       var = _iter_type( name = nomvar, atts = atts,
                         axes = list(axes.values()),
                         dtype = dtype,
                         record_id = record_id )
+      # Add auxiliary coordinate variables through silent dependency.
+      # Don't want these in the 'coordinates' attribute because they seem to
+      # cause issues with certain netCDF decoders.
+      if len(coords) > 0:
+        var.deps = coords
       self._varlist.append(var)
 
       #TODO: Find a minimum set of partial coverages for the data.
@@ -990,37 +878,18 @@ class BufferBase (object):
       for o in self._iter_objects(obj.deps,handled):
         yield o
 
-
-  # How to read the data.
-  # Override fstluk so it takes a record index instead of a key/handle.
-  def _fstluk (self, rec_id, dtype=None, rank=None, dataArray=None):
-    from rpnpy.librmn.fstd98 import fstluk
-    # Use local overrides for dtype.
-    if dtype is None and isinstance(rec_id,dict):
-      if 'datyp' in rec_id and 'nbits' in rec_id:
-        dtype = dtype_fst2numpy(rec_id['datyp'],rec_id['nbits'])
-
-    # Lock the opened file so another thread doesn't close it.
-    with self._lock:
-      if isinstance(rec_id,dict):
-        # If we were given a dict, assume it's for a valid open file.
-        key = rec_id['key']
-      else:
-        # Otherwise, need to open the file and get the proper key.
-        file_id = self._headers['file_id'][rec_id]
-        self._open(file_id)
-        key = int(self._headers['key'][rec_id]<<10) + self._opened_librmn_index
-
-      return fstluk (key, dtype, rank, dataArray)
-
   # How to decode the data from a raw binary array.
   def _decode (self, data, unused):
-    from fstd2nc.extra import decode
-    nbits = int(data[0x0b])
-    datyp = int(data[0x13])
-    dtype = dtype_fst2numpy(datyp, nbits)
-    out = decode(data).view(dtype)
-    return out
+    raise NotImplementedError("No decoder found.")
+
+  # Shortcuts to header decoding functions.
+  # Put into the class so they can potentially be overridden for other formats.
+  @staticmethod
+  def _decode_headers (headers):
+    raise NotImplementedError("No decoder found.")
+  @staticmethod
+  def _raw_headers (filename):
+    raise NotImplementedError("No decoder found.")
 
   #
   ###############################################
